import chimera, numpy
from chimera import runCommand
from numpy import zeros, array

from StructBio.Utilities.miscFunctions import residueKDhColor
from StructBio.Scenographics.solids import Solids
from StructBio.Utilities.shellNeighbors import Shell

#==============================================================================
#  ResidueCenter  Class
#==============================================================================
class ResidueCenter(object):
    def __init__(self, resObj):
        self.residue = resObj
        self.coords = zeros(3)

        # Note: self.coords will be the coordinates of the alpha carbon for GLY
        # and for any residue with a missing side chain (for example,
        # some end-of-chain residues in 1ZNJ)
        cbExists = self.residue.findAtom('CB')
        if not cbExists:
            self.coords = array(self.residue.findAtom('CA').coord())
        else:
            aCount = 0
            for a in self.residue.atoms:
                if a.name in ("CA", "C", "N", "O"): continue
                self.coords += array(a.coord())
                aCount += 1
            self.coords = self.coords/aCount

    def coord(self):
        return self.coords
    
#==============================================================================
# Mainline
#==============================================================================

resDistThreshold = 8.
runCommand("set bg_color white")

# Spheres for residues:
phobicSpheres = Solids("Phobic Spheres")
philicSpheres = Solids("Philic Spheres")
# Spindles for spheres:
phobic_phobic = Solids("Phobic-Phobic Spindles")
phobic_philic = Solids("Phobic-Philic Spindles")
philic_philic = Solids("Philic-Philic Spindles")


# Good examples to try: 1FNA, 1QCX, 2RHE, 1ZNJ
pdbID = raw_input("Type in four character PDB Identifierd: \n")

prot = chimera.openModels.open(pdbID, type = "PDB")[0]

resCntr_L = []
# Build a list of ResidueCenter objects and generate neighbor dictionary.
# We can also filter with respect to hydrophobicity.
for r in prot.residues:
    if not r.kdHydrophobicity: continue # Skip unusual residues not having a KD value.
    if r.findAtom("CA"):                # Checking for standard residue.
        resCntr_L.append(ResidueCenter(r))

rCshell = Shell(resCntr_L, 0.0, resDistThreshold)



# Put in spheres and spindles.
for rC in resCntr_L:
    if rC.residue.kdHydrophobicity > 0.0:
        phobicSpheres.addSphere(rC.coords, 0.7, residueKDhColor(rC.residue))
    else:
        philicSpheres.addSphere(rC.coords, 0.7, residueKDhColor(rC.residue))
        
    for nbrC in rCshell.getAtomsInShell(rC):
        # We do not want to put a spindle in twice:
        if nbrC.residue.id.chainId < rC.residue.id.chainId: continue
        if (nbrC.residue.id.chainId == rC.residue.id.chainId and
            nbrC.residue.id.position < rC.residue.id.position): continue
        spindColor = tuple((array(residueKDhColor(rC.residue)) +
                            array(residueKDhColor(nbrC.residue)))/2.0)
        if rC.residue.kdHydrophobicity > 0.0 and nbrC.residue.kdHydrophobicity > 0.0:
            phobic_phobic.addSpindle(rC.coords, nbrC.coords, 0.3, spindColor)
        if rC.residue.kdHydrophobicity * nbrC.residue.kdHydrophobicity < 0.0:
            phobic_philic.addSpindle(rC.coords, nbrC.coords, 0.3, spindColor)
        if rC.residue.kdHydrophobicity < 0.0 and nbrC.residue.kdHydrophobicity < 0.0:
            philic_philic.addSpindle(rC.coords, nbrC.coords, 0.3, spindColor)

phobicSpheres.display(); philicSpheres.display()
phobic_phobic.display(); phobic_philic.display(); philic_philic.display()




